using System;
using System.Data;
using System.Web.UI.WebControls;

using CMS.CMSHelper;
using CMS.FormEngine;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.UIControls;

public partial class CMSModules_CustomTables_Controls_CustomTableDataList : CMSUserControl
{
    #region "Private & protected variables"

    protected string editToolTip = String.Empty;
    protected string deleteToolTip = String.Empty;
    protected string viewToolTip = String.Empty;
    protected string upToolTip = String.Empty;
    protected string downToolTip = String.Empty;

    // Default pages
    private string mEditItemPage = "~/CMSModules/CustomTables/Tools/CustomTable_Data_EditItem.aspx";
    private string mViewItemPage = "~/CMSModules/CustomTables/Tools/CustomTable_Data_ViewItem.aspx";

    protected DataSet ds = null;
    private DataClassInfo mCustomTableClassInfo = null;
    private readonly CustomTableItemProvider ctProvider = new CustomTableItemProvider(CMSContext.CurrentUser);
    private bool hasItemOrderField = true;

    #endregion


    #region "Properties"

    /// <summary>
    /// Gets or sets URL of the page where item is edited
    /// </summary>
    public string EditItemPage
    {
        get
        {
            return mEditItemPage;
        }
        set
        {
            mEditItemPage = value;
        }
    }


    /// <summary>
    /// Gets or sets URL of the page where whole item is displayed
    /// </summary>
    public string ViewItemPage
    {
        get
        {
            return mViewItemPage;
        }
        set
        {
            mViewItemPage = value;
        }
    }


    /// <summary>
    /// Gets or sets the class info of custom table which data are displayed
    /// </summary>
    public DataClassInfo CustomTableClassInfo
    {
        get
        {
            return mCustomTableClassInfo;
        }
        set
        {
            mCustomTableClassInfo = value;
        }
    }

    #endregion


    /// <summary>
    /// Page load
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        // Register Javascripts
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);
        Page.ClientScript.RegisterClientScriptBlock(typeof(string), "DeleteEditView", ScriptHelper.GetScript(
           "var deleteConfirmation = ''; " +
           "function DeleteConfirm() { return confirm(deleteConfirmation); } " +
           "function EditItem(customtableid, itemId) { " +
           "  document.location.replace('" + ResolveUrl(EditItemPage) + "?customtableid=' + customtableid + '&itemId=' + itemId); } " +
           "function ViewItem(customtableid, itemId) { " +
           "  modalDialog('" + ResolveUrl(ViewItemPage) + "?customtableid=' + customtableid + '&itemId=' + itemId,'ViewItem',600,600); } "
           ));

        // Buttons' tooltips
        editToolTip = ResHelper.GetString("general.edit");
        deleteToolTip = ResHelper.GetString("general.delete");
        viewToolTip = ResHelper.GetString("general.view");
        upToolTip = ResHelper.GetString("general.up");
        downToolTip = ResHelper.GetString("general.down");

        // Delete confirmation
        ltlScript.Text = ScriptHelper.GetScript("deleteConfirmation = '" + ResHelper.GetString("customtable.data.DeleteConfirmation") + "';");

        gridData.RowDataBound += gridData_RowDataBound;
        gridData.GridLines = GridLines.Horizontal;

        if (CustomTableClassInfo != null)
        {
            string columns = CustomTableClassInfo.ClassShowColumns;
            if (string.IsNullOrEmpty(columns))
            {
                ds = ctProvider.GetItems(CustomTableClassInfo.ClassName, null, null);
            }
            else
            {
                // Ensure PK and order column
                columns = ";" + columns.Trim(';') + ";";
                if (!columns.ToLower().Contains(";itemid;"))
                {
                    columns = "ItemID" + columns;
                }
                if (!columns.ToLower().Contains(";itemorder;") && CustomTableClassInfo.ContainsColumn("itemorder"))
                {
                    columns = columns + "ItemOrder";
                }
                columns = columns.Trim(';').Replace(';', ',');

                ds = ctProvider.GetItems(CustomTableClassInfo.ClassName, null, null, -1, columns);
            }

            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                hasItemOrderField = ds.Tables[0].Columns.Contains("ItemOrder");

                if (hasItemOrderField)
                {
                    ds.Tables[0].DefaultView.Sort = "ItemOrder ASC";
                    gridData.DataSource = ds.Tables[0].DefaultView;
                }
                else
                {
                    gridData.DataSource = ds;
                }

                // Set displayed columns to the gridview
                if (!RequestHelper.IsPostBack())
                {
                    SetDisplayedColumns(ds, CustomTableClassInfo.ClassShowColumns);
                }
                gridData.DataBind();

            }
        }
    }


    private string EnsureColumn(string columnName, string columns)
    {
        return columns;
    }


    /// <summary>
    /// gridData page index changing event handler.
    /// </summary>
    protected void gridData_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        // Get sort expression
        string[] sortExp = ViewState["sorting"] as string[];
        if (sortExp != null)
        {
            DataSet dataSet = null;
            if (gridData.DataSource is DataSet)
            {
                dataSet = (DataSet)gridData.DataSource;
            }
            else
            {
                dataSet = ((DataView)gridData.DataSource).Table.DataSet;
            }
            dataSet.Tables[0].DefaultView.Sort = sortExp[0] + " " + sortExp[1];
            gridData.DataSource = dataSet.Tables[0].DefaultView;
        }
        // Change page index
        gridData.PageIndex = e.NewPageIndex;
        gridData.DataBind();
    }


    /// <summary>
    /// Binds command argument to the image buttons.
    /// </summary>
    protected void gridData_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        string commandArgument = "";
        if (e.Row.DataItem != null)
        {
            commandArgument = Convert.ToString(((DataRowView)(e.Row.DataItem))[0]);
        }

        ImageButton buttonDelete = ((ImageButton)(e.Row.FindControl("btnDelete")));
        if (buttonDelete != null)
        {
            buttonDelete.CommandArgument = commandArgument;
        }

        string customTableId = CustomTableClassInfo.ClassID.ToString();

        ImageButton buttonEdit = ((ImageButton)(e.Row.FindControl("btnEdit")));
        if (buttonEdit != null)
        {
            buttonEdit.OnClientClick = "EditItem(" + customTableId + ", " + Convert.ToString(((DataRowView)(e.Row.DataItem))[0]) + "); return false;";
            buttonEdit.CommandArgument = commandArgument;
        }

        ImageButton buttonView = ((ImageButton)(e.Row.FindControl("btnView")));
        if (buttonView != null)
        {
            buttonView.OnClientClick = "ViewItem(" + customTableId + ", " + Convert.ToString(((DataRowView)(e.Row.DataItem))[0]) + "); return false;";
            buttonView.CommandArgument = commandArgument;
        }

        ImageButton buttonUp = ((ImageButton)(e.Row.FindControl("btnUp")));
        if (buttonUp != null)
        {
            buttonUp.Visible = hasItemOrderField;
            buttonUp.CommandArgument = commandArgument;
        }

        ImageButton buttonDown = ((ImageButton)(e.Row.FindControl("btnDown")));
        if (buttonDown != null)
        {
            buttonDown.Visible = hasItemOrderField;
            buttonDown.CommandArgument = commandArgument;
        }
    }


    /// <summary>
    /// Set displayed columns to the gridview.
    /// </summary>
    /// <param name="dataSet">Dataset with gridview data.</param>
    /// <param name="reportFields">String with report fields separated by ';'</param>
    protected void SetDisplayedColumns(DataSet dataSet, string reportFields)
    {
        FormInfo fi = new FormInfo();
        string[] columnNames = null;

        if (CustomTableClassInfo != null)
        {
            // Get class form definition
            fi.LoadXmlDefinition(CustomTableClassInfo.ClassFormDefinition);

            // Get columns names            
            columnNames = fi.GetColumnNames();
        }

        // No field is set to be displayed -> set all fields to be dispalyed
        if (String.IsNullOrEmpty(reportFields))
        {
            if (columnNames != null)
            {
                foreach (string column in columnNames)
                {
                    reportFields += column + ";";
                }

                // Remove ending ';'
                if (reportFields != null)
                {
                    reportFields = reportFields.TrimEnd(';');
                }

                // Save columns
                CustomTableClassInfo.ClassShowColumns = reportFields;
                DataClassInfoProvider.SetDataClass(CustomTableClassInfo);
            }
        }

        if ((dataSet.Tables.Count > 0) && (dataSet.Tables[0].Columns.Count > 0))
        {
            if (reportFields != null)
            {
                reportFields = ";" + reportFields.ToLower().Trim(';') + ";";
            }

            // Go through the form fields
            if (columnNames != null)
            {
                foreach (string columnName in columnNames)
                {
                    if (reportFields != null)
                    {
                        if (reportFields.Contains(";" + columnName.ToLower() + ";") && (dataSet.Tables[0].Columns.Contains(columnName)))
                        {
                            // Get field caption
                            FormFieldInfo ffi = fi.GetFormField(columnName);
                            string fieldCaption = string.Empty;
                            if (ffi == null)
                            {
                                fieldCaption = columnName;
                            }
                            else
                            {
                                fieldCaption = (ffi.Caption == string.Empty) ? columnName : ResHelper.LocalizeString(ffi.Caption);
                            }

                            // Set column datafield and caption                    
                            BoundField column = new BoundField();
                            column.DataField = columnName;
                            column.HeaderText = fieldCaption;
                            column.ItemStyle.Wrap = false;
                            column.SortExpression = columnName;
                            column.HeaderStyle.Wrap = false;

                            // Add column to gridview
                            gridData.Columns.Add(column);
                        }
                    }
                }
            }
            // Set last column width to 100%
            if (gridData.Columns.Count > 1)
            {
                gridData.Columns[gridData.Columns.Count - 1].ItemStyle.Width = new Unit("100%");
            }

            // Set action column header text
            gridData.Columns[0].HeaderText = ResHelper.GetString("unigrid.actions");
        }
        else
        {
            // Hide action icons when no item is displayed
            gridData.Columns.Clear();
        }
    }


    /// <summary>
    /// Delete selected form data.
    /// </summary>
    protected void btnDelete_OnCommand(object sender, CommandEventArgs e)
    {
        // Check 'Delete' permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.customtables", "Delete") &&
            !CMSContext.CurrentUser.IsAuthorizedPerClassName(CustomTableClassInfo.ClassName, "Delete"))
        {
            lblError.Visible = true;
            lblError.Text = String.Format(ResHelper.GetString("customtable.permissiondenied.Delete"), CustomTableClassInfo.ClassName);
            return;
        }

        if (CustomTableClassInfo != null)
        {
            CustomTableItem item = ctProvider.GetItem(ValidationHelper.GetInteger(e.CommandArgument, 0), CustomTableClassInfo.ClassName);
            if (item != null)
            {
                item.Delete();
            }
        }

        UrlHelper.Redirect(Page.Request.Url.ToString());
    }


    /// <summary>
    /// Move item up
    /// </summary>
    protected void btnUp_OnCommand(object sender, CommandEventArgs e)
    {
        // Check 'Modify' permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.customtables", "Modify") &&
            !CMSContext.CurrentUser.IsAuthorizedPerClassName(CustomTableClassInfo.ClassName, "Modify"))
        {
            lblError.Visible = true;
            lblError.Text = String.Format(ResHelper.GetString("customtable.permissiondenied.Modify"), CustomTableClassInfo.ClassName);
            return;
        }

        if (CustomTableClassInfo != null)
        {
            ctProvider.MoveItemUp(ValidationHelper.GetInteger(e.CommandArgument, 0), CustomTableClassInfo.ClassName);
        }

        UrlHelper.Redirect(Page.Request.Url.ToString());
    }


    /// <summary>
    /// Move item down
    /// </summary>
    protected void btnDown_OnCommand(object sender, CommandEventArgs e)
    {
        // Check 'Modify' permission
        if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.customtables", "Modify") &&
            !CMSContext.CurrentUser.IsAuthorizedPerClassName(CustomTableClassInfo.ClassName, "Modify"))
        {
            lblError.Visible = true;
            lblError.Text = String.Format(ResHelper.GetString("customtable.permissiondenied.Modify"), CustomTableClassInfo.ClassName);
            return;
        }

        if (CustomTableClassInfo != null)
        {
            ctProvider.MoveItemDown(ValidationHelper.GetInteger(e.CommandArgument, 0), CustomTableClassInfo.ClassName);
        }

        UrlHelper.Redirect(Page.Request.Url.ToString());
    }


    /// <summary>
    /// gridData sort event hadler.
    /// </summary>
    protected void gridData_sort(object sender, GridViewSortEventArgs e)
    {
        string[] direction = ViewState["sorting"] as string[];
        if (direction == null)
        {
            direction = new string[] { e.SortExpression, "ASC" };
            ViewState["sorting"] = direction;
        }
        else
        {
            // Same column - reverse order
            if (e.SortExpression == direction[0])
            {
                direction = new string[] { e.SortExpression, (direction[1] == "ASC") ? "DESC" : "ASC" };
                ViewState["sorting"] = direction;
            }
            else
            {
                direction = new string[] { e.SortExpression, "ASC" };
                ViewState["sorting"] = direction;
            }
        }
        DataSet dataSet = null;
        if (gridData.DataSource is DataSet)
        {
            dataSet = (DataSet)gridData.DataSource;
        }
        else
        {
            dataSet = ((DataView)gridData.DataSource).Table.DataSet;
        }
        dataSet.Tables[0].DefaultView.Sort = e.SortExpression + " " + direction[1];
        gridData.DataSource = dataSet.Tables[0].DefaultView;
        gridData.DataBind();
    }
}
